#ifndef FITNESS_H
#define FITNESS_H

#include <eo>
#include <iostream>   // cout
#include <iomanip>

#include "ind_strategy.h" 

#include "compete.h" 


#include "strategies/kadlr3am_strategy.h"
#include "strategies/klara_strategy.h"
#include "strategies/knight.h"
#include "strategies/kucci.h"
#include "strategies/my_strategy.h"
#include "strategies/o_strategy.h"
#include "strategies/ondrova_strategy.h"
#include "strategies/prokop_strategy.h"
#include "strategies/strategyPP.h"
#include "strategies/tomik_strategy.h"
#include "strategies/PavelN.h"




#define NUM_STRATEGIES 21


using namespace std; 

typedef eoBit<double> Indi; 

class PrisonerDilemaFitness : public eoPopEvalFunc<Indi> {

public:

  PrisonerDilemaFitness::PrisonerDilemaFitness();
  Strategy* PrisonerDilemaFitness::createStrategy(const string& name);

  void operator()(eoPop<Indi> & _parents, eoPop<Indi> & _offspring) {

    rewards.resize( _offspring.size() );
    for (unsigned i=0; i<_offspring.size(); i++) 
      rewards[i] = 0;
    
    compete.setTrials(100,0);
    compete.printOff();


    for (unsigned i=0; i<_offspring.size(); i++) {
      IndiStrategy me(_offspring[i]);
      
      for (unsigned j=0; j<i; j++) {
	IndiStrategy op(_offspring[j]); 
	result res = compete( me, op );
	rewards[i] += res.first_player;
	rewards[j] += res.second_player;
      } 
    }
    for (unsigned i=0; i<_offspring.size(); i++) 
      _offspring[i].fitness( rewards[i] / (_offspring.size()-1) ); 
  } 

private:
  Compete compete;
  vector<double> rewards;


public: // not used in fitness comp
  vector<string> stratNames;
  vector<bool>  stratEnabled;
  vector<Strategy*>  targetStrategies; 

};  

PrisonerDilemaFitness::PrisonerDilemaFitness() {

  stratNames.push_back("RandomStrategy");
  stratNames.push_back("AlwaysCooperate");
  stratNames.push_back("AlwaysDeceive");
  stratNames.push_back("TFT_(CopyPartner)");
  stratNames.push_back("Alternate");
  stratNames.push_back("Pavlov");
  stratNames.push_back("Spite");
  stratNames.push_back("CCD");
  stratNames.push_back("DDC");
  stratNames.push_back("FQuick");
  stratNames.push_back("TF2T");
  
  // strategie ze cviceni 
  stratNames.push_back("Pavel_Nohejl");
  stratNames.push_back("kadlr3am_Strategie");
  stratNames.push_back("KlarinaStrategie");
  stratNames.push_back("Knight");
  stratNames.push_back("Kucci");
  stratNames.push_back("OndrovaStrategie");
  stratNames.push_back("Ondrova_Super_Strategie");
  stratNames.push_back("ProkopStrategy");
  stratNames.push_back("Strategie_PP");
  stratNames.push_back("Tomikova_strategie"); 

  if (NUM_STRATEGIES != stratNames.size()) 
    throw std::runtime_error("Too many or not enough names of strategies.");

  stratEnabled.resize(NUM_STRATEGIES); 
  for (unsigned i=0; i<stratEnabled.size(); i++)
    stratEnabled[i] = true;

  targetStrategies.resize(NUM_STRATEGIES);
  for (unsigned i=0; i<stratEnabled.size(); i++)
    targetStrategies[i] = createStrategy(stratNames[i]);
  

} 

Strategy* PrisonerDilemaFitness::createStrategy(const string& name){
  if (name == "RandomStrategy") return new RandomStrategy(); 
  if (name == "AlwaysCooperate") return new AlwaysCooperate(); 
  if (name == "AlwaysDeceive") return new AlwaysDeceive();
  if (name == "TFT_(CopyPartner)") return new CopyPartner();
  if (name == "Alternate") return new Alternate();
  if (name == "Pavlov") return new Pavlov();
  if (name == "Spite") return new Spite();
  if (name == "CCD") return new CCD();
  if (name == "DDC") return new DDC();
  if (name == "FQuick") return new FQuick();
  if (name == "TF2T") return new TF2T();
  
  if (name == "Pavel_Nohejl") return new PavelN();
  if (name == "kadlr3am_Strategie") return new KadlecStrategy();
  if (name == "KlarinaStrategie") return new KStrategy();
  if (name == "Knight") return new Knight();
  if (name == "Kucci") return new Kucci();

  if (name == "OndrovaStrategie") return new OStrategy();
  if (name == "Ondrova_Super_Strategie") return new OndrovaStrategie();
  if (name == "ProkopStrategy") return new ProkopStrategy();
  if (name == "Strategie_PP") return new StrategyPP();
  if (name == "Tomikova_strategie") return new Tomik();

  throw std::runtime_error("unknown strategy: "+name);
  return 0;
}  


#endif
